#!/usr/bin/perl -w
# squish.cgi  --  See why you've been squished
#
# Copyright (c) 2001-2002 Andrew McGill and Leading Edge Business Solutions
# (South Africa).  This software may be redistributed and/or modified only
# under the terms of the GNU General Public Licence, version 2, as published by
# the Free Software Foundation, and contained in the file COPYING. 
# 
# All other rights are reserved, and no warranty is implied.  Despite any
# appearance to the contrary, this software is not fit for ANY purpose you may
# conceive, and should not be used on your system under any circumstances.
# 
# This program reads a squid log from stdin and creates a time and data based
# blocklist for use with squid

#use English;
use lib '/var/www/cuotas';

$revision = '$Id: squish.cgi,v 1.18 2019/03/20 05:18:56 andrewm Exp $';
$revision =~ s/(.*,v | \S+ \S+ \$$)//g;

# $DEBUG=1;

no warnings qw{uninitialized};
use POSIX qw(strftime mktime);

use CGI qw/:standard/;
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);
use rrdsquish;

do "squish.pm";
#use Storable qw( lock_store lock_retrieve );

$graphheight=150;
$graphwidth=350;

# Get the user RRD for bytes or time
sub userrrd($$)
{
	my $user = shift;
	my $type = shift;
	return $$userdb{$user}{$type};
}

sub showuser($)
{
	my $user=shift;
	my $rrd_time = $$userdb{$user}{"time"};
	my $rrd_data = $$userdb{$user}{"data"};

	my $timetoascii = strftime ("%a %d %b %Y %H:%M", localtime($rrd_time->{timestamp}));
	printf STDOUT "%-15s $timetoascii  %5d %9d\n",
		$user,
		$rrd_time->sum(0,24*3600),
		$rrd_data->sum(0,24*3600);

	#print $rrd_time->print(0,3600*24*2);

	return;

	print STDOUT "$user: time (m:10m:h:d) ".
		$rrd_time->normalise(60,0,59).":".
		$rrd_time->normalise(60,0,600-1).":".
		$rrd_time->normalise(60,0,3600-1).":".
		$rrd_time->normalise(60,0,86400-1).
		$rrd_time->normalise(60,0,86400*30-1).
		" data (m:5m:h:d) ".
		$rrd_data->sum(0,59).":".
		$rrd_data->sum(0,600).":".
		$rrd_data->sum(0,3600).":".
		$rrd_data->sum(0,86400)."\n";
	print $rrd_time->print(60,3600*24*2);
	# 250ms delay
	select(undef, undef, undef, 0.25);
}


sub showdata()
{
	print "Name/IP         Last access             time      data\n";
	foreach $user (sort keys %$userdb) {
		showuser($user);
	}
}

sub min($$)
{
	my $x=shift, $y=shift;
	if ($x<$y) { return $x; }
	return $y;
}

sub cvtime($)
{
	my $t=shift;
	my @r;
	if ($t<1) {
		@r=('-',"" ,"");
	}
	elsif ($t<60) {
		@r=($t," second". ($t==1?"":"s") ,"s");
	}
	elsif ($t<3600) {
		@r=(sprintf('%d:%02d',int($t/60),$t%60)," minutes","m");
	}
	elsif ($t<86400*2) {
		@r=(sprintf('%d:%02d',$t/3600,($t%3600)/60)," hours","h");
	}
	elsif ($t<86400*31) {
		$t=$t/86400;
		@r= (sprintf('%d',$t),int($t)==1 ? " day":" days","d");
	}
	else {
		$t=$t/86400/30.5;
		@r= (sprintf('%d',$t),$t==1 ? " month":" months","mo");
	}
	return wantarray ? @r : $r[0].$r[2];
}

sub cvdata($)
{
	my $t=shift;
	my @r;
	if ($t==0) {
		@r= ($t,"","");
	}
	elsif ($t<1024) {
		@r= (sprintf('%d',$t),"bytes","b");
	}
	elsif ($t<1024*1024) {
		@r= (sprintf('%.2f',$t/1024),"kbytes","kb");
	}
	elsif ($t<1024*1024*1024) {
		@r= (sprintf('%.2f',$t/1024/1024),"mbytes","mb");
	}
	else {
		@r= (sprintf('%.3f',$t/1024/1024/1024),"Gbytes","Gb");
	}
	return wantarray ? @r : $r[0].$r[2];
}

sub drawgraph($$$)
{
	my $user=shift;
	my $type=shift;
	my $t=shift;

	use GD;

 	# Make it
	my $im = new GD::Image($graphwidth,$graphheight);

	my $white = $im->colorAllocate(255,255,255);
	my $grey  = $im->colorAllocate(0xD0,0xD0,0xD0);
	my $rxcol = $im->colorAllocate(0x40,0x99,0x40);
	my $txcol = $im->colorAllocate(0x99,0x40,0x40);
	
	my $bgcol = $im->colorAllocate(255,255,255);
	my $black = $im->colorAllocate(0,0,0);
	my $bordercol = $im->colorAllocate(196,196,255);
 
	my $title;
	if ($type eq 'data') {
		$cv = \&cvdata; 
		$title="Bandwidth usage";
		$drawcol = $txcol;
	}
	elsif ($type eq 'time') {
		$cv = \&cvtime;
		$title="Time on-line";
		$drawcol = $rxcol;
	}
	else { return; }

	$type =~ m/^(data|time)$/ or return;

	my @x=();
	my @y=();
	my $rrd = userrrd($user,$type);
	if (!defined $rrd) {return; }
	my ($total,$max) = $rrd->getgraph($t, \@x, \@y);
	if (!defined $total) {return; }
	my ($total_s,$total_u,$total_uu)=&$cv($total);
	my @ticks = ();
	if ($t == 0) { @ticks = (1800,3600,3600*2,3600*3,3600*4,3600*5,3600*6,3600*9,3600*12,3600*18); }
	elsif ($t == 1) { @ticks = (86400*1,86400*2,86400*3,86400*4,86400*5,86400*6); }
	elsif ($t == 2) { @ticks = (86400*1,86400*7,86400*14,86400*21,86400*28); }
	elsif ($t == 3) { @ticks = (2635200,2635200*2,2635200*3,2635200*6,2635200*9); }

	my $xx=@x;  # number of items
 	if ($xx<1) { $xx=1; }

	# Parameters
	my $hoffset=16;
	my $voffset= 16;
	my $width=$graphwidth - $hoffset-1;
	my $height=$graphheight - $voffset - 16;
	if ($max<0.00001) { $max=10; };
	my $bar = ($width-2)/$xx;
	my $unit = $height / $max;

	if (1) {
		# smooth it;
		#for ($i=0;$i<$xx;$i++) { $ii[$i]=($y[$i]+$y[$i+1])/2; } @y=@ii;
		#for ($i=0;$i<$xx;$i++) { $ii[$i]=($y[$i]+$y[$i+1])/2; } @y=@ii;
	}
 
 	$im->filledRectangle($hoffset, $voffset, $width+$hoffset, $voffset+$height, $bgcol);
 	# Horizontal lines
 	$im->rectangle($hoffset,$voffset+$height/2,$hoffset+$width-1,$voffset+$height/4*3,$bordercol);
	my $a;
	for $a (@ticks) {
 		$left = $hoffset + ( $a/$x[1] * $bar);
 		$im->line(
 			$left, $voffset,
 			$left, $voffset + $height,
 			$grey);
	}
 	for ($a = 0; $a < $xx; $a++) {
 		# RX: green, TX: push
 		$left = $hoffset + ( $a * $bar);
		my $h=$voffset + $height - min($height, $unit * $y[$a]);
 		$im->filledRectangle(
 			$left, $h,
 			$left+1, $voffset + $height,
 			$drawcol);
 	}
	$title .= " ($total_s$total_u)";
 	$im->string(gdSmallFont,$hoffset,1,$title,$black);
	my ($step_s,$step_u,$step_uu)=cvtime($x[1]);
 	$im->string(gdSmallFont,$hoffset,$voffset+$height+2,"Step: $step_s $step_u",$black);
	my ($max_s,$max_u,$max_uu)=cvtime($x[$xx-1]+$x[1]);
 	$im->string(gdSmallFont,$hoffset+$width-7*9-2,$voffset+$height+2,
		"$max_s$max_u",$black);
 	$im->rectangle($hoffset,$voffset,$hoffset+$width-1,$voffset+$height+1,$bordercol);
 	$im->stringUp(gdSmallFont,0,$voffset+$height,"0",$black);
	my ($max_s,$max_u,$max_uu)=&$cv($max);
 	$im->stringUp(gdSmallFont,0,$voffset+48,"$max_s$max_uu",$black);
 
	print header("image/png"),
		$im->png;
}

# Make an HTML title
sub title
{
	my $text=shift;

        #header("Date: " . gmdate("D, d M Y H:i:s") . " GMT");           // alway
	#header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); // always

	return header(-pragma=>"no-cache",
		-cache_control=>"no-cache, must-revalidate",
		-expires=>"Mon, 26 Jul 1997 05:00:00 GMT",
		-type=>"text/html")."<!doctype html>
<html lang='en'>
  <head>
    <meta charset='utf-8'>
    <meta name='viewport' content='width=device-width, initial-scale=1, shrink-to-fit=no'>
    <meta name='description' content='>
    <meta name='author' content='>
    <link rel='icon' href='favicon.ico'>

    <title>Cuotas</title>

    <!-- Bootstrap core CSS -->
    <link href='css/bootstrap.min.css' rel='stylesheet'>
    <link href='css/font-awesome.min.css' rel='stylesheet'>

    <!-- Custom styles for this template -->
    <link href='css/custom.css' rel='stylesheet'>
  </head>

  <body>

  <header>
    <nav class='navbar navbar-main fixed-top navbar-dark'>
      <div class='container'>
        <div class='col-12 text-primary text-center'><h3></h3></div>
      </div>
    </nav>
  </header>
  
  <section class='container'>
  
    <div class='row border-bottom'>
      <h3>$text</h3>
    </div>";
}

sub footer()
{
return "</section>
  <!-- END CONTENIDO -->
  <footer>
    <nav class='navbar navbar-footer fixed-bottom navbar-dark'>
      <div class='container'>
        <div class='col-12 text-center text-muted'><small>&copy; 2019 EEPF Estaci&oacute;n Indio Hatuey, Perico,Matanzas, rev $revision</small></div>
        <div class='col-12 text-center text-muted'><small><i class='fa fa-map-marker'></i> Direccin:  Central Espana Republicana #318. Matanzas. Cuba.
        <i class='fa fa-phone'></i> (45) 57-1226</small></div>
      </div> 
    </nav>
  </footer>
    <!-- Bootstrap core JavaScript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    <!--<script src='https://code.jquery.com/jquery-3.2.1.slim.min.js' integrity='sha384-KJ3o2DKtIkvYIK3UENzmM7KCkRr/rE9/Qpg6aAZGJwFDMVNA/GpGFF93hXpG5KkN' crossorigin='anonymous'></script>-->
    <!--<script>window.jQuery || document.write('<script src='js/vendor/jquery-slim.min.js'><\/script>')</script>-->
    <script src='js/vendor/popper.min.js'></script>
    <script src='js/bootstrap.min.js'></script>
    <script src='js/jquery.js'></script>
  </body>
</html>"	
}

sub linktographs($$$)
{
	# showgraph($user,"Last 10 minutes",600,30);
	my $user = shift;
	my $title = shift;
	my $class = shift;
	print "<p><i>$title</i><br>\n";
	my $link="?pic&u=$user&type=data&t=$class$demotext";
	print "<a href=$link><img width=$graphwidth height=$graphheight src=\"$link\"></a>&nbsp;";
	my $link="?pic&u=$user&type=time&t=$class$demotext";
	print "<a href=$link><img width=$graphwidth height=$graphheight src=\"$link\"></a>&nbsp;";
}

sub withflair($$)
{
my $a=shift;
my $b=shift;
if ($sortindex<4 && $sortindex >= 0) {
	$a=$$userdb{$a}{"t:data"}[$sortindex];
	$b=$$userdb{$b}{"t:data"}[$sortindex];
	$b <=> $a;
}
elsif ($sortindex<8) {
	$a=$$userdb{$a}{"t:time"}[$sortindex-4];
	$b=$$userdb{$b}{"t:time"}[$sortindex-4];
	$b <=> $a;
}
else {
	$a cmp $b;
}

}


# MAIN

#if ( defined param("squished") ) {
#	print title("Quota exceeded") .
#	"You have exceeded your quota, either in terms of the amount of\n".
#	"data downloaded via this proxy, or in terms of the time you have\n".
#	"spent browsing.\n".
#	"<p>\n".
#	"Please consult your administrator for further information.  If you\n".
#	"are the administrator, you may modify <i>/etc/squid3/squish.conf</i>\n".
#	"then run <i>/usr/local/squish/squish.cron.sh</i> to set allocations.\n".
#	"<p>\n".
#	footer();
#	exit;
#}
if ( defined param("squished") ) {
	print title("Cuota exedida") .
	"Usted ha consumido su cuota de navegacin, en trminos de datos descargados a traves del proxy.\n".
	"<p>\n".
	"Por favor consulte a su administrador para mas informacin. \n".
	"<p>\n".
	footer();
	exit;
}


getcurrentstate();

# If this is a demo, then cut the number of records in the database ...
$demo=0;
$demotext="";
if ( defined param("demo") ) {
	$demo=1;
	$demotext="&demo";
	$democount=4;
	$sortindex=0;
	foreach $user (sort withflair keys %$userdb) {
		if ($democount-- <= 0) {
			$blah = delete $$userdb{$user};
			print "$user";
		}
	}
}

if ( defined param("pic") ) {
	# print "<img src=\"?pic&u=$user&duration=$duration&interval=$interval><p>\n";
	drawgraph(param("u"),param("type"),param("t"));
}
elsif ( defined param("dump") ) {
	# print "<img src=\"?pic&u=$user&duration=$duration&interval=$interval><p>\n";
	my $user = param("u");
	print title("dump $user");
	my $rrd = userrrd($user,"time");
	print "<pre>".$rrd->dump()."</pre>";
	footer();
}
elsif ( defined param("u") ) {
	my $user = param("u");
	my $rrd = userrrd($user,"data");
	my $timetoascii = strftime ("%a %d %b %Y %H:%M",
		localtime($rrd->{timestamp})); 

	print title("Datos del usuario $user [$timetoascii]");
	my $reason = $$userdb{$user}{"reason"};
	if ($reason ne '') {
		print "<font color=red>Squished:</font> $reason<br>";
	}
	linktographs($user,"Last 24 hours",0);
	linktographs($user,"Week",1);
	linktographs($user,"Month",2);
	linktographs($user,"Year",3);
	print footer();
}
else {


print title("Uso del Proxy en Indio Hatuey");
print "<div class='row mt-2'>\n";
print "Seleccionar usuario para ver detalles, listado de sitios libres de CUOTAS&nbsp;<a href=\"/no-quota.html\" target=\"_tab\" style=\"color: red\">AQUI.</a>\n";
print "<table class='table table-striped table-bordered table-hover table-sm rounded'>\n";
print "<thead class='thead-main'>\n";
print "<tr>
            <th scope='col' class='text-white'>Usuario</th>
            <th scope='col' class='text-center table-link'><a href=\"?i=0$demotext\">24&nbsp;horas</a></th>
            <th scope='col' class='text-center table-link'><a href=\"?i=1$demotext\">Semana</a></th>
            <th scope='col' class='text-center table-link'><a href=\"?i=2$demotext\">Mes</a></th>
            <th scope='col' class='text-center table-link'><a href=\"?i=4$demotext\">24&nbsp;horas</a></th>
            <th scope='col' class='text-center table-link'><a href=\"?i=5$demotext\">Semana</a></th>
            <th scope='col' class='text-center table-link'><a href=\"?i=6$demotext\">Mes</a></th>
          </tr>\n";
print "</thead>\n";
print "<tbody>\n";

$sortindex=param("i");
foreach $user (sort withflair keys %$userdb) {
	if(($user != "" )||($user eq "giraldo" )||($user eq "adonis1" )||($user eq "tania" )){
	}
	else{
	my $reason = $$userdb{$user}{"reason"};
	my $font = '';
	my $unfont = '';
	if ($reason ne '') {
		$font="<font color=red>";
		$unfont="</font>";
	}
#	print "<tr><td><a href=\"?u=$user$demotext\">$font$user$unfont</a></td>\n";

	print "<tr><td><a href=\"?u=$user$demotext\">$font$user$unfont</a> - <small><a href=\"http://proxy.ihatuey.cu/squidlog/squint/all/daily/usage-$user.html\" target=\"_tab\" style=\"font-size: 85%;color: green;\"> Reportes</a><small></td>\n";
	print "<td align=right>".cvdata($$userdb{$user}{"t:data"}[0])."</td>\n";
	print "<td align=right>".cvdata($$userdb{$user}{"t:data"}[1])."</td>\n";
	print "<td align=right>".cvdata($$userdb{$user}{"t:data"}[2])."</td>\n";
#	print "<td align=right>".cvdata($$userdb{$user}{"t:data"}[3])."</td>\n";
	print "<td align=right>".cvtime($$userdb{$user}{"t:time"}[0])."</td>\n";
	print "<td align=right>".cvtime($$userdb{$user}{"t:time"}[1])."</td>\n";
	print "<td align=right>".cvtime($$userdb{$user}{"t:time"}[2])."</td>\n";
#	print "<td align=right>".cvtime($$userdb{$user}{"t:time"}[3])."</td>\n";
	print "</tr>\n";

	foreach my $n ("t:data","t:time") {
		foreach my $i (0,1,2,3) {
			$ttl{$n}[$i]+=$$userdb{$user}{$n}[$i];
		}
	}
      	}
	
}
print "</tbody>\n";

print "<thead class='thead-main'>\n";
print "<tr><th>Total</th>\n";
print "<th class='text-center'>".cvdata($ttl{"t:data"}[0])."</th>\n";
print "<th class='text-center'>".cvdata($ttl{"t:data"}[1])."</th>\n";
print "<th class='text-center'>".cvdata($ttl{"t:data"}[2])."</th>\n";
print "<th class='text-center'>".cvtime($ttl{"t:time"}[0])."</th>\n";
print "<th class='text-center'>".cvtime($ttl{"t:time"}[1])."</th>\n";
print "<th class='text-center'>".cvtime($ttl{"t:time"}[2])."</th>\n";
print "</tr>\n";
print "</thead>\n";
print "</table>\n";

print footer();
}


